<?php

namespace Drupal\translation_inheritance;

use Drupal\Core\Entity\ContentEntityInterface;
use Drupal\Core\Security\TrustedCallbackInterface;
use Drupal\Core\Url;

/**
 * Defines some helper functions and callbacks for the translation inheritance
 * functionality.
 */
class TranslationInheritance implements TranslationInheritanceInterface, TrustedCallbackInterface {

  /**
   * Shows a message on the layout builder page if the translation is inherited.
   */
  public static function alterLayoutBuilder($element) {
    $request = \Drupal::request();
    if (!$entity_type_id = $request->attributes->get('entity_type_id')) {
      return $element;
    }

    $entity = $request->attributes->get($entity_type_id);
    if (!$entity instanceof ContentEntityInterface) {
      return $element;
    }


    // The entity has to have the field for this to be applicable.
    if (!$entity->hasField(static::FIELD_NAME)) {
      return $element;
    }

    // If this is the default translation, or this translation simply doesn't
    // inherit from any other language, we instantly return.
    if (!$entity->get(TranslationInheritanceInterface::FIELD_NAME)->source_language) {
      return $element;
    }

    // Translation inheritance can be recursive, make sure to get the actual
    // entity we need to render.
    while ($source_language = $entity->get(TranslationInheritanceInterface::FIELD_NAME)->source_language) {
      $entity = $entity->getTranslation($source_language);
    }

    $language = $entity->language();
    $url = Url::createFromRequest($request)->setOption('language', $language);
    \Drupal::messenger()->addWarning(t('This layout will not be shown because this translation inherits its content from the @source-language.
To edit that content, please <a href=":link">click here</a>.', [
      '@source' => $language->getId(),
      ':link' => $url->toString(),
    ]));

    return $element;
  }

  /**
   * {@inheritDoc}
   */
  public static function trustedCallbacks() {
    return ['alterLayoutBuilder'];
  }

}
